package com.muyu.event.process.consumer;

import com.muyu.event.process.basic.EventPublisher;
import com.muyu.event.process.event.IoTDBInsertDataEvent;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextClosedEvent;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/**
 * @Author: zi run
 * @Date 2024/9/29 23:23
 * @Description 车辆消费者
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class VehicleConsumer implements ApplicationRunner, ApplicationListener<ContextClosedEvent> {

    /**
     * kafka消费者
     */
    private final KafkaConsumer<String, String> kafkaConsumer;

    /**
     * 事件发布者
     */
    private final EventPublisher eventPublisher;

    /**
     * 协议解析报文传递数据(队列名称)
     */
    public final static String MESSAGE_PARSING = "MessageParsing";

    /**
     * 设定固定大小的线程池，线程数量与当前可用的处理器核心数相同
     */
    private final ExecutorService executorService =
            Executors.newFixedThreadPool(10);

    @Override
    public void run(ApplicationArguments args) throws Exception {
        log.info("启动线程监听Topic: {}", MESSAGE_PARSING);
        List<String> topics = Collections.singletonList(MESSAGE_PARSING);
        kafkaConsumer.subscribe(topics);
        while (true) {
            ConsumerRecords<String, String> consumerRecords = kafkaConsumer.poll(Duration.ofMillis(100));
            consumerRecords.forEach(consumerRecord -> executorService.submit(() -> handleRecord(consumerRecord)));
        }
    }

    private void handleRecord(ConsumerRecord<String, String> consumerRecord) {
        String message = consumerRecord.value();
        log.info("接收到车辆报文数据，内容：{}", message);
        log.info("------------------------------------------------");
        eventPublisher.publish(new IoTDBInsertDataEvent(this, message));
    }

    @Override
    public void onApplicationEvent(ContextClosedEvent event) {
        log.info("关闭线程池和Kafka消费者");

        try {
            executorService.shutdown();
            if (!executorService.awaitTermination(60, TimeUnit.SECONDS)) {
                executorService.shutdownNow();
            }
        } catch (InterruptedException e) {
            log.error("线程池关闭被中断，强制关闭", e);
            executorService.shutdownNow();
            Thread.currentThread().interrupt();
        }

        try {
            kafkaConsumer.close(); // 关闭Kafka消费者
        } catch (Exception e) {
            log.error("关闭Kafka消费者时发生错误", e);
        }
    }
}
